/* 1803, Sun 25 Nov 01 (PST)

   PCAP_CYGWD.C: System-specific functions for the Cygwin/WinDump
      port of libpcap.  Note that lookupdev() and lookupnet() have
      been moved from inet.c to here, where we have the includes
      they need.

   Copyright (C) 1992-2002 by Nevil Brownlee,
   CAIDA & The University of Auckland */

/*
 * Copyright (c) 1994, 1995, 1996
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static const char rcsid[] =
    "@(#) $Header: pcap-null.c,v 1.7 96/12/10 23:15:01 leres Exp $ (LBL)";
#endif

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/utsname.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "pcap-int.h"
#include "pcap-cygwd.h"


int pcap_setbuff(pcap_t *p, int dim)  /* Only for WinDump's packet.dll */
{
   if (!cygwd_setbuff(p->fd, dim)) {
      strcpy(p->errbuf,
         "pcap_setbuff(): not enough memory to allocate buffer");
      return -1;
      }
   return 0;
   }

int pcap_stats(pcap_t *p, struct pcap_stat *ps)
{
   struct bpf_stat s;
   if (!cygwd_stats(p->fd, &s)) {
      strcpy(p->errbuf,
         "pcap_stats(): couldn't get stats");
      return -1;
      }
   ps->ps_recv = s.bs_recv;
   ps->ps_drop = s.bs_drop;
   return 0;
   }

int pcap_setfilter(pcap_t *p, struct bpf_program *fp)
{
   if (!cygwd_have_adaptor(p->fd))
      p->fcode = *fp;  /* No adaptor, just remember the bpf program */
   if (!cygwd_setfilter(p->fd, fp)) {
      strcpy(p->errbuf,
         "pcap_stats(): couldn't set bpf filter");
      return -1;
      }
   return 0;
   }

pcap_t *pcap_open_live(
   char *device, int snaplen, int promisc, int to_ms, char *ebuf)
{
   pcap_t *p;
   int fd;

   if ((p = (pcap_t *)malloc(sizeof(pcap_t))) == NULL) {
      sprintf(ebuf, "malloc: %s", pcap_strerror(errno));
      return NULL;
      }
   memset(p, 0, sizeof(pcap_t));

   if ((fd = cygwd_open_adapter(device)) < 0) {
      strcpy(ebuf,"pcap_open_live(): couldn't open adapter");
      return NULL;
      }
   else p->fd = fd;

   cygwd_set_hwfilter(fd, promisc);

   if (!cygwd_get_nettype(fd, &p->linktype))
      strcpy(ebuf,"pcap_open_live(): couldn't determine network type");
   else {
      p->buffer = (u_char *)malloc(p->bufsize = RecBufSize);
      if (p->buffer == NULL)
         strcpy(ebuf,"pcap_open_live(): couldn't alloc pcap buffer");
      else {
         p->snapshot = snaplen;  /* packet.dll doesn't use this !!! */
         switch (cygwd_init_adaptor(fd, p->buffer, p->bufsize, to_ms)) {
         case CYF_PAP:
            strcpy(ebuf,"pcap_open_live(): couldn't init struct PACKET");
            break;
         case CYF_PSB:
            strcpy(ebuf,"pcap_open_live(): couldn't alloc packet buffer");
            break;
         case CYF_PST:
            strcpy(ebuf,"pcap_open_live(): couldn't set read timeout");
            break;
         case CYF_POK:
            return p;
            }
         }
      }

   cygwd_close_adaptor(fd);
   if (p->buffer != NULL) free(p->buffer);
   free(p);
   return NULL;
   }

int pcap_read(pcap_t *p, int cnt, pcap_handler callback, u_char *user)
{
   int cc;
   int n = 0;
   u_char *bp, *ep;

   cc = p->cc;
   if (p->cc == 0) {
      if (!cygwd_receive_packet(p->fd, &cc)) {
         strcpy(p->errbuf, "pcap_read(): PacketReceivePacket failed");
         return -1;
         }
      bp = p->buffer;
      }
   else bp = p->bp;

#define bhp ((struct bpf_hdr *)bp)
   ep = bp + cc;
   while (bp < ep) {
      int caplen, hdrlen;
      caplen = bhp->bh_caplen;
      hdrlen = bhp->bh_hdrlen;

      /* XXX A bpf_hdr matches a pcap_pkthdr */

      (*callback)(user, (struct pcap_pkthdr*)bp, bp + hdrlen);
      bp += BPF_WORDALIGN(caplen + hdrlen);
      if (++n >= cnt && cnt > 0) {
         p->bp = bp;
         p->cc = ep - bp;
         return n;
         }
      }
#undef bhp

   p->cc = 0;
   return n;
   }

#if NON_UNIX_NETIF

void pcap_close_fd(pcap_t *p)
{
   cygwd_close_adaptor(p->fd);
   }

/*
 * Return the name of a network interface attached to the system, or NULL
 * if none can be found.  The interface must be configured up; the
 * lowest unit number is preferred; loopback is ignored.
 */

char *pcap_lookupdev(char *errbuf)
{
   int ni, j;
   char *np;
   if ((ni = find_interfaces()) == 0)
      return NULL;  /* No WinDump-capable interfaces */
   for (j = 0; j != ni; ++j) {
      np = if_names[j];
      if (tolower(np[0]) != 'p' ||  /* Try to find non-PPP interface */
            tolower(np[1]) != 'p' || tolower(np[2]) != 'p')
         break;
      }
   if (j == ni) j = 0;  /* Only PPP interfaces! */
   return if_names[j];
   }

/* Find IP network number and mask for specified device;
   this function isn't used by any routines within libpcap.
   Since packet.dll doesn't give us the IP address for an
   interface, we just get hostname and look up its address.
   At least this works as expected for a single-interface host! */

int pcap_lookupnet(char *device,
   bpf_u_int32 *netp, bpf_u_int32 *maskp, char *errbuf)
{
   char to_host[100], *tp;
   struct hostent *hp;
   unsigned char *ap;  u_long a;

   if (gethostname(to_host, sizeof(to_host)) != 0) {
      strcpy(errbuf, "pcap_lookupnet couldn't get hostname");
      return -1;
      }
   if ((hp = gethostbyname(to_host)) == NULL) {
      strcpy(errbuf, "pcap_lookupnet couldn't get host address");
      return -1;
      }

   ap = &hp->h_addr_list[0][0];
   a = ap[0]<<24 | ap[1]<<16 | ap[2]<<8 | ap[3];
   if ((a & 0x80000000) == 0)  /* Got host IP address, now guess netmask */
      *maskp = 0xFF000000;  /* Class A */
   else if ((a & 0xC0000000) == 0x80000000)
      *maskp = 0xFFFF0000;  /* Class B */
   else if ((a & 0xE0000000) == 0xC0000000)
      *maskp = 0xFFFFFF00;  /* Class C */
   else {
      strcpy(errbuf, "pcap_lookupnet couldn't get netmask");
      return -1;
      }
   *netp = a & *maskp;
   return 0;
   }

#endif  /* NON_UNIX_NETIF */
